##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

__all__ = ('Bucket', 'Set', 'BTree', 'TreeSet',
           'LFBucket', 'LFSet', 'LFBTree', 'LFTreeSet',
           'union', 'intersection', 'difference',
           'weightedUnion', 'weightedIntersection', 'multiunion',
          )

from zope.interface import moduleProvides

from .Interfaces import IIntegerFloatBTreeModule
from ._base import Bucket
from ._base import MERGE
from ._base import MERGE_WEIGHT_numeric
from ._base import MERGE_DEFAULT_float
from ._base import Set
from ._base import Tree as BTree
from ._base import TreeSet
from ._base import _TreeIterator
from ._base import difference as _difference
from ._base import intersection as _intersection
from ._base import multiunion as _multiunion
from ._base import set_operation as _set_operation
from ._base import to_long as _to_key
from ._base import to_float as _to_value
from ._base import union as _union
from ._base import weightedIntersection as _weightedIntersection
from ._base import weightedUnion as _weightedUnion
from ._base import _fix_pickle
from ._compat import import_c_extension

_BUCKET_SIZE = 120
_TREE_SIZE = 500
using64bits = True


class LFBucketPy(Bucket):
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class LFSetPy(Set):
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class LFBTreePy(BTree):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class LFTreeSetPy(TreeSet):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class LFTreeIteratorPy(_TreeIterator):
    pass


# Can't declare forward refs, so fix up afterwards:

LFBucketPy._mapping_type = LFBucketPy._bucket_type = LFBucketPy
LFBucketPy._set_type = LFSetPy

LFSetPy._mapping_type = LFBucketPy
LFSetPy._set_type = LFSetPy._bucket_type = LFSetPy

LFBTreePy._mapping_type = LFBTreePy._bucket_type = LFBucketPy
LFBTreePy._set_type = LFSetPy

LFTreeSetPy._mapping_type = LFBucketPy
LFTreeSetPy._set_type = LFTreeSetPy._bucket_type = LFSetPy


differencePy = _set_operation(_difference, LFSetPy)
unionPy = _set_operation(_union, LFSetPy)
intersectionPy = _set_operation(_intersection, LFSetPy)
multiunionPy = _set_operation(_multiunion, LFSetPy)
weightedUnionPy = _set_operation(_weightedUnion, LFSetPy)
weightedIntersectionPy = _set_operation(_weightedIntersection, LFSetPy)

import_c_extension(globals())

_fix_pickle(globals(), __name__)

moduleProvides(IIntegerFloatBTreeModule)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
from BTrees.OOBTree import OOBTree, OOBucket

class B(OOBucket):
    pass

class T(OOBTree):
    _bucket_type = B
    max_leaf_size = 2
    max_internal_size = 3

class S(T):
    pass

import unittest

class SubclassTest(unittest.TestCase):

    def testSubclass(self):
        # test that a subclass that defines _bucket_type gets buckets
        # of that type
        t = T()
        t[0] = 0
        self.assertTrue(t._firstbucket.__class__ is B)

    def testCustomNodeSizes(self):
        # We override btree and bucket split sizes in BTree subclasses.
        t = S()
        for i in range(8):
            t[i] = i
        state = t.__getstate__()[0]
        self.assertEqual(len(state), 5)
        sub = state[0]
        self.assertEqual(sub.__class__, S)
        sub = sub.__getstate__()[0]
        self.assertEqual(len(sub), 5)
        sub = sub[0]
        self.assertEqual(sub.__class__, B)
        self.assertEqual(len(sub), 1)

def test_suite():
    return unittest.makeSuite(SubclassTest)



print("import: 'BTrees'")
import BTrees

print("import: 'BTrees._IFBTree'")
import BTrees._IFBTree

print("import: 'BTrees._IIBTree'")
import BTrees._IIBTree

print("import: 'BTrees._IOBTree'")
import BTrees._IOBTree

print("import: 'BTrees._LFBTree'")
import BTrees._LFBTree

print("import: 'BTrees._LLBTree'")
import BTrees._LLBTree

print("import: 'BTrees._LOBTree'")
import BTrees._LOBTree

print("import: 'BTrees._OIBTree'")
import BTrees._OIBTree

print("import: 'BTrees._OLBTree'")
import BTrees._OLBTree

print("import: 'BTrees._OOBTree'")
import BTrees._OOBTree

print("import: 'BTrees._fsBTree'")
import BTrees._fsBTree




##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
# Copied from ZODB/utils.py

from binascii import hexlify

from ._compat import _bytes

def non_negative(int_val):
    if int_val < 0:
        # Coerce to non-negative.
        int_val &= 0x7FFFFFFFFFFFFFFF
    return int_val


def positive_id(obj): #pragma NO COVER
    """Return id(obj) as a non-negative integer."""
    return non_negative(id(obj))


def oid_repr(oid):
    if isinstance(oid, _bytes) and len(oid) == 8:
        # Convert to hex and strip leading zeroes.
        as_hex = hexlify(oid).lstrip(b'0')
        # Ensure two characters per input byte.
        chunks = [b'0x']
        if len(as_hex) & 1:
            chunks.append(b'0')
        elif as_hex == b'':
            as_hex = b'00'
        chunks.append(as_hex)
        return b''.join(chunks)
    else:
        return repr(oid)



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest


class Test_non_negative(unittest.TestCase):

    def _callFUT(self, int_val):
        from BTrees.utils import non_negative
        return non_negative(int_val)

    def test_w_big_negative(self):
        self.assertEqual(self._callFUT(-(2**63 - 1)), 1)

    def test_w_negative(self):
        self.assertEqual(self._callFUT(-1), 2**63 - 1)

    def test_w_zero(self):
        self.assertEqual(self._callFUT(0), 0)

    def test_w_positive(self):
        self.assertEqual(self._callFUT(1), 1)

    def test_w_big_positive(self):
        import sys
        try:
            self.assertEqual(self._callFUT(sys.maxint), sys.maxint)
        except AttributeError: #pragma NO COVER Py3k
            pass


class Test_oid_repr(unittest.TestCase):

    def _callFUT(self, oid):
        from BTrees.utils import oid_repr
        return oid_repr(oid)

    def test_w_non_strings(self):
        self.assertEqual(self._callFUT(None), repr(None))
        self.assertEqual(self._callFUT(()), repr(()))
        self.assertEqual(self._callFUT([]), repr([]))
        self.assertEqual(self._callFUT({}), repr({}))
        self.assertEqual(self._callFUT(0), repr(0))

    def test_w_short_strings(self):
        for length in range(8):
            faux = 'x' * length
            self.assertEqual(self._callFUT(faux), repr(faux))

    def test_w_long_strings(self):
        for length in range(9, 1024):
            faux = 'x' * length
            self.assertEqual(self._callFUT(faux), repr(faux))

    def test_w_zero(self):
        self.assertEqual(self._callFUT(b'\0\0\0\0\0\0\0\0'), b'0x00')

    def test_w_one(self):
        self.assertEqual(self._callFUT(b'\0\0\0\0\0\0\0\1'), b'0x01')

    def test_w_even_length(self):
        self.assertEqual(self._callFUT(b'\0\0\0\0\0\0\xAB\xC4'), b'0xabc4')

    def test_w_odd_length(self):
        self.assertEqual(self._callFUT(b'\0\0\0\0\0\0\x0D\xEF'), b'0x0def')


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(Test_non_negative),
        unittest.makeSuite(Test_oid_repr),
    ))



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from .common import BTreeTests
from .common import ExtendedSetTests
from .common import I_SetsBase
from .common import InternalKeysMappingTest
from .common import InternalKeysSetTest
from .common import MappingBase
from .common import MappingConflictTestBase
from .common import ModuleTest
from .common import MultiUnion
from .common import NormalSetTests
from .common import SetConflictTestBase
from .common import SetResult
from .common import TestLongIntKeys
from .common import TestLongIntValues
from .common import Weighted
from .common import itemsToSet
from .common import makeBuilder


class LLBTreeInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBTree
        return LLBTree


class LLBTreePyInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBTreePy
        return LLBTreePy


class LLTreeSetInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSet
        return LLTreeSet


class LLTreeSetPyInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSetPy
        return LLTreeSetPy


class LLBucketTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBucket
        return LLBucket


class LLBucketTestPy(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBucketPy
        return LLBucketPy


class LLTreeSetTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSet
        return LLTreeSet


class LLTreeSetTestPy(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSetPy
        return LLTreeSetPy


class LLSetTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLSet
        return LLSet


class LLSetTestPy(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLSetPy
        return LLSetPy


class LLBTreeTest(BTreeTests, TestLongIntKeys, TestLongIntValues,
                  unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLBTree
        return LLBTree()
    def getTwoValues(self):
        return 1, 2


class LLBTreeTestPy(BTreeTests, TestLongIntKeys, TestLongIntValues,
                  unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLBTreePy
        return LLBTreePy()
    def getTwoValues(self):
        return 1, 2


class TestLLSets(I_SetsBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLSet
        return LLSet()


class TestLLSetsPy(I_SetsBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLSetPy
        return LLSetPy()


class TestLLTreeSets(I_SetsBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLTreeSet
        return LLTreeSet()


class TestLLTreeSetsPy(I_SetsBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LLBTree import LLTreeSetPy
        return LLTreeSetPy()


class PureLL(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.LLBTree import union
        return union(*args)

    def intersection(self, *args):
        from BTrees.LLBTree import intersection
        return intersection(*args)

    def difference(self, *args):
        from BTrees.LLBTree import difference
        return difference(*args)

    def builders(self):
        from BTrees.LLBTree import LLBTree
        from BTrees.LLBTree import LLBucket
        from BTrees.LLBTree import LLTreeSet
        from BTrees.LLBTree import LLSet
        return LLSet, LLTreeSet, makeBuilder(LLBTree), makeBuilder(LLBucket)


class PureLLPy(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.LLBTree import unionPy
        return unionPy(*args)

    def intersection(self, *args):
        from BTrees.LLBTree import intersectionPy
        return intersectionPy(*args)

    def difference(self, *args):
        from BTrees.LLBTree import differencePy
        return differencePy(*args)

    def builders(self):
        from BTrees.LLBTree import LLBTreePy
        from BTrees.LLBTree import LLBucketPy
        from BTrees.LLBTree import LLTreeSetPy
        from BTrees.LLBTree import LLSetPy
        return (LLSetPy, LLTreeSetPy,
                makeBuilder(LLBTreePy), makeBuilder(LLBucketPy))


class TestLLMultiUnion(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.LLBTree import multiunion
        return multiunion(*args)

    def union(self, *args):
        from BTrees.LLBTree import union
        return union(*args)

    def mkset(self, *args):
        from BTrees.LLBTree import LLSet as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.LLBTree import LLTreeSet as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.LLBTree import LLBucket as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.LLBTree import LLBTree as mkbtree
        return mkbtree(*args)


class TestLLMultiUnionPy(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.LLBTree import multiunionPy
        return multiunionPy(*args)

    def union(self, *args):
        from BTrees.LLBTree import unionPy
        return unionPy(*args)

    def mkset(self, *args):
        from BTrees.LLBTree import LLSetPy as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.LLBTree import LLTreeSetPy as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.LLBTree import LLBucketPy as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.LLBTree import LLBTreePy as mkbtree
        return mkbtree(*args)


class TestWeightedLL(Weighted, unittest.TestCase):

    def weightedUnion(self):
        from BTrees.LLBTree import weightedUnion
        return weightedUnion

    def weightedIntersection(self):
        from BTrees.LLBTree import weightedIntersection
        return weightedIntersection

    def union(self):
        from BTrees.LLBTree import union
        return union

    def intersection(self):
        from BTrees.LLBTree import intersection
        return intersection

    def mkbucket(self, *args):
        from BTrees.LLBTree import LLBucket as mkbucket
        return mkbucket(*args)

    def builders(self):
        from BTrees.LLBTree import LLBTree
        from BTrees.LLBTree import LLBucket
        from BTrees.LLBTree import LLTreeSet
        from BTrees.LLBTree import LLSet
        return LLBucket, LLBTree, itemsToSet(LLSet), itemsToSet(LLTreeSet)


class TestWeightedLLPy(Weighted, unittest.TestCase):

    def weightedUnion(self):
        from BTrees.LLBTree import weightedUnionPy
        return weightedUnionPy

    def weightedIntersection(self):
        from BTrees.LLBTree import weightedIntersectionPy
        return weightedIntersectionPy

    def union(self):
        from BTrees.LLBTree import unionPy
        return unionPy

    def intersection(self):
        from BTrees.LLBTree import intersectionPy
        return intersectionPy

    def mkbucket(self, *args):
        from BTrees.LLBTree import LLBucketPy as mkbucket
        return mkbucket(*args)

    def builders(self):
        from BTrees.LLBTree import LLBTreePy
        from BTrees.LLBTree import LLBucketPy
        from BTrees.LLBTree import LLTreeSetPy
        from BTrees.LLBTree import LLSetPy
        return (LLBucketPy, LLBTreePy,
                itemsToSet(LLSetPy), itemsToSet(LLTreeSetPy))


class LLBTreeConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBTree
        return LLBTree


class LLBTreeConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBTreePy
        return LLBTreePy


class LLBucketConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBucket
        return LLBucket


class LLBucketConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLBucketPy
        return LLBucketPy


class LLTreeSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSet
        return LLTreeSet


class LLTreeSetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLTreeSetPy
        return LLTreeSetPy


class LLSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLSet
        return LLSet


class LLSetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LLBTree import LLSetPy
        return LLSetPy


class LLModuleTest(ModuleTest, unittest.TestCase):

    prefix = 'LL'

    def _getModule(self):
        import BTrees
        return BTrees.LLBTree

    def _getInterface(self):
        import BTrees.Interfaces
        return BTrees.Interfaces.IIntegerIntegerBTreeModule


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(LLBTreeInternalKeyTest),
        unittest.makeSuite(LLBTreeInternalKeyTest),
        unittest.makeSuite(LLTreeSetInternalKeyTest),
        unittest.makeSuite(LLTreeSetInternalKeyTest),
        unittest.makeSuite(LLBucketTest),
        unittest.makeSuite(LLBucketTest),
        unittest.makeSuite(LLTreeSetTest),
        unittest.makeSuite(LLTreeSetTest),
        unittest.makeSuite(LLSetTest),
        unittest.makeSuite(LLSetTest),
        unittest.makeSuite(LLBTreeTest),
        unittest.makeSuite(LLBTreeTest),
        unittest.makeSuite(TestLLSets),
        unittest.makeSuite(TestLLSets),
        unittest.makeSuite(TestLLTreeSets),
        unittest.makeSuite(TestLLTreeSets),
        unittest.makeSuite(TestLLMultiUnion),
        unittest.makeSuite(TestLLMultiUnion),
        unittest.makeSuite(PureLL),
        unittest.makeSuite(PureLL),
        unittest.makeSuite(TestWeightedLL),
        unittest.makeSuite(TestWeightedLL),
        unittest.makeSuite(LLBTreeConflictTests),
        unittest.makeSuite(LLBTreeConflictTests),
        unittest.makeSuite(LLBucketConflictTests),
        unittest.makeSuite(LLBucketConflictTests),
        unittest.makeSuite(LLTreeSetConflictTests),
        unittest.makeSuite(LLTreeSetConflictTests),
        unittest.makeSuite(LLSetConflictTests),
        unittest.makeSuite(LLSetConflictTests),
        unittest.makeSuite(LLModuleTest),
    ))
